<?php

// Relative theme values.
define('HEX_CURRENT_PAGE', 0);
define('HEX_ACTIVE',       1);
define('HEX_ALL_ACTIVE',   2);
define('HEX_CASCADE',      3);

/**
 * Returns the current active theme. Use this instead of the global variable.
 *
 * @param $get
 *  When set to HEX_CURRENT_PAGE, this will always return the theme used for
 *  the rendered page. When set to HEX_ACTIVE, it will return the internally
 *  active theme. For example, if the site has a different admin theme,
 *  setting this to HEX_ACTIVE will return the site wide active theme while
 *  HEX_CURRENT_PAGE will return the admin theme used to display the page.
 *  When inside the theme settings api form, the internal focus will be on the
 *  theme specific form so HEX_ACTIVE will point to the theme associated with
 *  that form.
 */
function hex_active_theme($get = HEX_ACTIVE) {
  switch ($get) {
    case HEX_ACTIVE       : return $GLOBALS['theme_key'];
    case HEX_CURRENT_PAGE : return $GLOBALS['theme'];
  }
}

/**
 * List the active themes starting from parents.
 *
 * @see hex_active_theme()
 *
 * @param $get
 *  See the explanation on this same parameter in hex_active_theme(). The list
 *  of returned themes can depend on various factors.
 */
function hex_active_themes($get = HEX_ACTIVE) {
  static $theme_lists;

  $theme = hex_active_theme($get);
  if (!isset($theme_lists[$theme])) {
    $theme_lists[$theme] = array();

    $themes = list_themes();
    $_theme = $theme;

    while (isset($_theme)) {
      $theme_lists[$theme][$_theme] = $_theme;
      $_theme = isset($themes[$_theme]->base_theme) ? $themes[$_theme]->base_theme : NULL;
    }
    $theme_lists[$theme] = array_reverse($theme_lists[$theme]);
  }

  return $theme_lists[$theme];
}

/**
 * Get the values defined within the .info file.
 * 
 * @param $info_key
 *  (required) The key to retrieve.
 * @param $default
 *  Fall back value if nothing is found.
 * @param $theme
 *  Theme specific value. If not set, it will return the value for the active
 *  theme.
 */
function hex_theme_info($info_key, $default = NULL, $theme = HEX_ACTIVE) {

  if ($theme === HEX_ACTIVE) {
    $theme = hex_active_theme();
  }
  $list_themes = list_themes();

  return isset($list_themes[$theme]->info[$info_key]) ? $list_themes[$theme]->info[$info_key] : $default;
}

/**
 * Returns information about a theming hook.
 */
function hex_theme_hook_info($hook) {
  static $hooks;

  if (!isset($hooks)) {
    $hooks = theme_get_registry();
  }

  return isset($hooks[$hook]['hex_info']) ? $hooks[$hook]['hex_info'] : NULL;
}

/**
 * A more flexible version of core's theme_get_setting().
 *
 * The setting is obtained from the first value found in the following sources:
 * - the saved values from the theme's settings form
 * - the saved values from the global theme settings form
 * - the default theme-specific settings defined in the theme's .info file
 *
 * The list above will cycle through each connected base and sub-themes with
 * sub-themes having priority for their values when the $source parameter is
 * set to HEX_CASCADE (default). Core's theme_get_setting() is designed to
 * fall back automatically but it's currently broken.
 *
 * @see http://api.drupal.org/api/search/7/theme_get_setting
 * @see http://drupal.org/node/761608
 *  core issue: inconsistent theme data
 *
 * @param $name
 *  The name of the setting to be retrieved. This works like domains for
 *  settings structured in an array. Selecting the top level setting will
 *  return the whole array. Traversing children can be done with a dot for
 *  more specific data, e.g., 'parent_setting.sub_setting.sub_etc'. There are
 *  also conditional parent scopes where the setting does not enclose the
 *  child. Instead, the parent setting will be checked for non-empty values
 *  before the child is exposed of its own value. A working example of this is
 *  in the 'development_mode' setting which is a simple boolean switch. When
 *  it is turned off, 'development_mode:disable_theme_caching' will not return
 *  a result even if it is enabled.
 * @param $default
 *  The default value if nothing was found.
 * @param $source
 *  (string|const) Can be a theme key which will restricted it to a specific
 *  theme. The HEX_ACTIVE constant restricts it to the running theme and
 *  HEX_CASCADE will look into all themes and pick the value from the child
 *  most theme.
 */
function hex_setting($name = NULL, $default = NULL, $source = HEX_CASCADE) {
  $cache = _hex_cache('settings');

  if ($source === HEX_ACTIVE) {
    $source = hex_active_theme();
  }
  elseif ($source == HEX_CASCADE && !empty($cache->cascade[$name])) {
    $source = $cache->cascade[$name];
  }

  return isset($cache->settings[$source][$name]) ? $cache->settings[$source][$name] : $default;
}

/**
 * Retrieve multiple settings from multiple sources or "scopes".
 *
 * Example:
 *
 *  $settings = hex_settings(array(
 *    SETTING_KEY => SCOPE,
 *    'key_1'     => HEX_ACTIVE,
 *    'key_2'     => HEX_ALL_ACTIVE,
 *    'key_3'     => HEX_CASCADE,
 *    'key_4'     => THEMENAME,
 *  ));
 *
 * TODO: document this further.
 *
 * @param $setting_keys
 *  Array of setting keys and their scope.
 */
function hex_settings($setting_keys) {
  $cache = _hex_cache('settings');

  foreach ($setting_keys as $setting_key => $scope) {
    $settings[$setting_key] = $scopes = array();

    if ($scope == HEX_ALL_ACTIVE) {
      $scopes = hex_active_themes();
    }
    elseif ($scope == HEX_ACTIVE) {
      $scopes[] = hex_active_theme();
    }
    elseif ($scope == HEX_CASCADE && isset($cache->cascade[$setting_key])) {
      $scopes[] = $cache->cascade[$setting_key];
    }
    elseif (in_array($scope, hex_active_themes())) {
      $scopes[] = $scope;
    }

    foreach ($scopes as $scope) {
      if (isset($cache->settings[$scope][$setting_key])) {
        $settings[$setting_key][] = array('setting' => $cache->settings[$scope][$setting_key],
                                          'scope'   => $scope);
      }
    }
  }

  return $settings;
}

/**
 * Pass in a type to get a list of active components. This function is useful
 * when accessing multiple data types in single way. This will always return a
 * flat array.
 *
 * @param $type
 *  Possible values: 'theme', 'plugin', 'module' or 'library'.
 * @param $name
 *  Only used when $type is plugin or library. The $name can be the plugin set
 *  or the module hosting a library.
 */
function hex_list($type, $name = NULL) {
  switch ($type) {
    case 'theme'  : return hex_active_themes();
    case 'plugin' : return hex_plugin_list($name);
    case 'module' : return module_list();
    case 'library':
      if ($list = array_keys(drupal_get_library($name))) {
        return array_combine($list, $list);
      }
  }
}

/**
 * Provides a standard way for retrieving paths to themes, plugins, etc...
 * 
 * @see drupal_get_path()
 * 
 * @param $type
 *  Anything accepted by drupal_get_path() in addition to 'plugin'.
 * @param $name
 *  Name of the item to retrieve.
 */
function hex_path($type, $name) {
  switch ($type) {
    case 'plugin': return hex_plugin_path($name);
    default:       return drupal_get_path($type, $name);
  }
}

/**
 * Provides status and logs them for theme developers.
 *
 * Users granted 'administer themes' permission will be shown a status
 * message. Notices should be minimal and be shown only to relevant roles.
 *
 * When the $type is 'debug', the theme setting 'development_mode' and
 * 'development_mode:debug_log' must be enabled for the active theme. Debug
 * entries will not be shown on the front-end. They will go straight to the
 * watchdog log.
 *
 * @see drupal_set_message()
 * @see watchdog()
 *
 * @param $message
 *  The message text. Works the same way as watchdog() but it is also used
 *  for the status text.
 * @param $variables
 *  Placeholder variables for $messages.
 * @param $type
 *  The level of the message. Can only be one of four possible values:
 *  error, warning, status or debug.
 */
function hex_notify($message, $variables = array(), $type = 'status') {
  static $status;

  if (!isset($status)) {
    if (user_access('administer themes')) {
      $status = array('error'   => WATCHDOG_ERROR,
                      'warning' => WATCHDOG_WARNING,
                      'status'  => WATCHDOG_INFO);
    }
    if (hex_setting('development_mode:debug_log', 0, HEX_ACTIVE)) {
      $status['debug'] = WATCHDOG_DEBUG;
    }
  }

  if (isset($status[$type])) {
    if ($type !== 'debug') {
      drupal_set_message(t($message, $variables), $type, FALSE);
    }
    watchdog('hexagon', $message, $variables, $status[$type]);
  }
}
